/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


static char const rcsid[] =
"$Id: av-sophie.c,v 1.4 2004/01/30 15:02:33 lwa Exp $";

#include "config.h"

#include <syslog.h>

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "lmtp.h"

#define SIGNATURE "scanned by Sophie, http://www.vanja.com/tools/"
#define SOPHIE_BUFLEN 1020

static void av_sophie_close(int fd) {
  write(fd, "QUIT\n", sizeof("QUIT\n")-1);
}


static int av_sophie_scan(struct fstate *state) {
  char buf[SOPHIE_BUFLEN];
  char *s;
  int fd;
  int len;

  len = snprintf(buf, sizeof(buf), "%s.rfc822/%lu\n", state->myhostname, 
		 (unsigned long)mailwrite_size(state, state->hdr_my_received,
					       HEADER_OTHER));
  if (len >= sizeof(buf)) {
    syslog(LOG_INFO, "Sophie query too large\n");
    return -1;
  }

  fd = rxconnect_test(&state->av.sophie, buf, len);
  if (fd == -1)
    return -1;

  if ((len == read(fd, buf, sizeof(buf))) == -1) {
    roll_server(&state->av.sophie, "IO error in Sophie server %s: %s",
                current_server(&state->av.sophie), strerror(errno));
    return -1;
  }
  if (len < 3 || buf[0] != 'O' || buf[1] != 'K') {
    if (len) buf[len-1] = 0; /* remove \n */
    roll_server(&state->av.sophie, "Sophie server %s not OK: %s",
                current_server(&state->av.sophie), strerror(errno));
    return -1;
  }
  if (mailwrite(state, state->hdr_my_received, HEADER_OTHER, fd) == -1 ||
      (len = read(fd, buf, sizeof(buf))) == -1) {
    roll_server(&state->av.sophie, "IO error in Sophie server %s: %s",
                current_server(&state->av.sophie), strerror(errno));
    return -1;
  }
  buf[len] = 0;
  if ((s=memchr(buf, '\n', len)) != NULL)
    *s = 0;

  rxclose(&state->av.sophie);
  
  if (buf[0] == '1') {
#define INFECTED SIGNATURE "\n\tinfected with "
    char outbuf[sizeof(buf) + sizeof(INFECTED) + 2];
    sprintf(outbuf, INFECTED "%s", buf+2);
    av_set_virus(state, "Sophie", buf+2);
    state->av.desc = strdup(outbuf);
    return 1;
  } else if (buf[0] == '-' && buf[1] == '1') {
    syslog(LOG_INFO, ID_FMT "Unknown Sophie status %s", state->counter, buf);
    return -1;
  } else {
    state->av.desc = strdup(SIGNATURE);
    return 0;
  }
}

av_scanner_t av_sophie_init(struct fstate *state) {
  rxinit(&state->av.sophie, 1, av_sophie_close);
  return av_sophie_scan;
}
