/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: av.c,v 1.13 2003/12/02 12:07:38 lwa Exp $";
#include "config.h"

#include <sys/time.h>

#include <syslog.h>

#include <strings.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "lmtp.h"

int av_message(struct fstate *state, char *path) {
  FILE *msg, *tmp;
  char domain[255];
  char tmpname[PATH_MAX];
  int c;

  getuserdomain(state->qrecipient, domain);

  if ((msg = opentemplate(path, domain))==NULL)
    return -1;
  if ((tmp=newmessage(state, "av", tmpname))==NULL) {
    fclose(msg);
    return -1;
  }
  while((c=fgetc(msg))!=EOF) {
    if (c=='%') {
      c=fgetc(msg);
      switch(c) {
      case 'D':
	fputs(domain, tmp);
	break;
      case 'R':
        fputs(state->qrecipient, tmp);
        break;
      case 's': {
        char stamp[100];
        strftime(stamp, sizeof(stamp)-1,
                 "%Y%m%d%H%M%S", localtime(&(state->date)));
        fputs(stamp, tmp);
	break;
      }
      case 'F': {
	struct headers *hl = state->hdrlist;
	while(hl) {
	  if (hl->type == HEADER_FROM) {
	    char *from=hl->header+hl->dd_len+1;
	    while(isspace((int)*from)) from++;
	    fputs(from, tmp);
	    break;
	  }
	  hl = hl->next;
	}
	break;
      }
      case 'S': {
	struct headers *hl = state->hdrlist;
	while(hl) {
	  if (hl->type == HEADER_SUBJECT) {
	    char *subject=hl->header+hl->dd_len+1;
	    while(isspace((int)*subject)) subject++;
	    fputs(subject, tmp);
	    break;
	  }
	  hl = hl->next;
	}
	break;
      }
      case 'v':
	fputs(state->av.virus, tmp);
	break;
      case 'H': {
	ssize_t bodysize = state->bodysize;
	fflush(tmp);
	state->bodysize = 0;
	mailwrite(state, state->hdrlist, HEADER_OTHER,
		  fileno(tmp));
	state->bodysize = bodysize;
	break;
      }
      case 'B':
	fflush(tmp);
	write(fileno(tmp), state->body, state->bodysize);
	break;
      case EOF:
	break;
      default:
	fputc(c, tmp);
      }
    } else {
      fputc(c, tmp);
    }
  }
  fclose(msg);
  if (endmessage(state, "av", tmpname, tmp) == EOF) {
    return -1;
  }
  return 0;
}

#define ADDRESS        "255.255.255.255"
#define CLIENT_STRING  ", smtp_client=" 

void av_set_virus(struct fstate *state, char *antivirus, char *virus) {
  struct timeval end_time;
  double scan_time;
  char client[sizeof(CLIENT_STRING ADDRESS)];
  int ip[4];
  if (get_addr_from_received(state->hdr_in_received->header, ip,
			     client+sizeof(CLIENT_STRING)-1, NULL, NULL) > 0) {
    memcpy(client, CLIENT_STRING, sizeof(CLIENT_STRING)-1);
  } else {
    client[0] = 0;
  }
  gettimeofday(&end_time, NULL);
  scan_time = (double)(end_time.tv_sec - state->av.start_time.tv_sec) + 
    (double)(end_time.tv_usec - state->av.start_time.tv_usec) / 1000000;
    
  syslog(LOG_INFO, ID_FMT "[%s] infected with %s (scan_time=%.2fs%s)",
	 state->counter, antivirus, virus, scan_time, client);
  snprintf(state->av.bounce_message, sizeof(state->av.bounce_message),
	   "infected with %s", virus);
  state->av.bounce_message[sizeof(state->av.bounce_message) - 1] = 0;
  strncpy(state->av.virus, virus, sizeof(state->av.virus) - 1);
  state->av.virus[sizeof(state->av.virus) - 1] = 0;
}

static int av_null_scan(struct fstate *state) {
  return 0;
}

void av_reset(struct fstate *state) {
  state->av.nvirus = -1;
  state->av.scanned = 0;
  state->av.bounce_message[0] = 0;
  state->av.virus[0] = 0;
  gettimeofday(&state->av.start_time, NULL);
  if (state->av.desc) {
    free(state->av.desc);
    state->av.desc = NULL;
  }
}

int av_init(struct fstate *state) {
  if (state->av.scan == NULL) {
    if (state->av.type == NULL) {
      state->av.scan = av_null_scan;
    } else if (strcasecmp(state->av.type, "drweb") == 0) {
      state->av.scan = av_drweb_init(state);
    } else if (strcasecmp(state->av.type, "clamd") == 0) {
      state->av.scan = av_clamd_init(state);
    } else if (strcasecmp(state->av.type, "sophie") == 0) {
      state->av.scan = av_sophie_init(state);
    } else {
      state->av.scan = av_null_scan;
    }
  }
  if (state->av.viruskeeper.user) {
    if (state->av.viruskeeper.user[0] == 0) {
      free(state->av.viruskeeper.user);
      state->av.viruskeeper.user = NULL;
    } else if (getrcpt(state, &state->av.viruskeeper) == NULL) {
      syslog(LOG_ERR, "virus keeper user <%s> not found",
	     state->av.viruskeeper.user);
      free(state->av.viruskeeper.user);
      state->av.viruskeeper.user = NULL;
    }
  }
  return 0;
}

int av_scan(struct fstate *state) {
  if (!state->av.scanned) {
    av_reset(state);
    state->av.nvirus = state->av.scan(state);
    if (state->av.desc) {
      unsigned char *s = state->av.desc;
      while(*s) {
	if (s[0] == '\n') {
	  if (s[1] == 0)
	    s[0] = 0;
	  else if (s[1] != '\t' && s[1] != ' ')
	    s[0] = ' ';
	} else if (*s == '\r') {
	  *s = ' ';
	} else if (*s > 127) {
	  *s = '?';
	}
	s++;
      }
    }
    state->av.scanned = 1;
  }
  if (state->av.bounce_message[0])
    state->user_bounce_message = state->av.bounce_message;
  return state->av.nvirus;
}

