/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: checker.c,v 1.51 2004/04/20 13:16:53 lwa Exp $";

#include "config.h"

#include <stdio.h>
#include <sys/types.h>
#include <stdarg.h>
#include <string.h>
#include <locale.h>
#include <stdlib.h>
#include <nl_types.h>
#include <ctype.h>

#ifndef PARSER
#define PARSER
#endif

#include "lmtp.h"

nl_catd catd;
#define CGS(y, s) catgets(catd, 1, y, s)

static char line[1024];
static int lineno=0;
static int haserror=0;
static int haserror_on_line=0;

void parser_nextline(char *buf) {
  char *ebuf;
  lineno++;
  haserror_on_line=0;
  strncpy(line, buf, sizeof(line)-1);
  if ((ebuf=strchr(line, '\n'))!=NULL) {
    *ebuf=0;
  }
}

int doerr(char *fmt, ...) {
  va_list ap;
  if (haserror_on_line)
    return 0;
  va_start(ap, fmt);
  fprintf(stderr, CGS(1, "Line %d (%s)"), lineno, line);
  fputc('\n', stderr);
  fprintf(stderr, CGS(2, "Error:"), lineno, line);
  fputc(' ', stderr);
  vfprintf(stderr, fmt, ap);
  fputc('\n', stderr);
  fputc('\n', stderr);
  va_end(ap);
  haserror++;
  haserror_on_line++;
  return 0;
}

int checkrelay(struct fstate *state) {
  return 0;
}

int spam_assassin(struct fstate *state, char *a) {
  return 0;
}

char *seterr(struct fstate *state, int code, char *ecode, char *fmt, ...) {
  doerr(CGS(7, "error in parser"));
  return NULL;
}

int av_scan(struct fstate *state) {
  return 0;
}

int valid_re(RE *re) {
  switch(re->err) {
  case RE_ERR_MEMORY:
    doerr(CGS(16, "memory allocation"));
    return -1;
  case RE_ERR_SYNTAX:
    if (re->errstr) {
      doerr(CGS(18, "invalid expression: %s"), re->errstr);
    } else {
      doerr(CGS(17, "invalid regexp declaration"));
    }
    return -1;
  }
  return 0;
}

int regchk(RE *re, char *hdr, int sense,
	   struct headers *hdrlist, int private) {
  return 1;
}

int bodchk(struct fstate *state, RE *re, int len) {
  return 1;
}

int modchk(struct fstate *state, RE *re, char *module, int sense) {
  if (state->in_module) {
    doerr(CGS(22, "call of modules not allowed in module"));
  } else if (*module == 0 || *module == '.' || strchr(module, '/')) {
    doerr(CGS(15, "invalid module name (%s)"), module);
  }
  return 1;
}

int cmd_deliver(struct fstate *state, char *a) {
  char *box=strdup(a);
  getbox(a);
  if (strcmp(a, box)!=0) {
    doerr(CGS(9, "bad box name \"%s\""), box);
  }
  free(box);
  return FILTER_CONTINUE;
}

int cmd_deliver2(struct fstate *state, char *a) {
  return cmd_deliver(state, a);
}

int cmd_archive(struct fstate *state, char *a) {
  return cmd_deliver(state, a);
}

int cmd_archive2(struct fstate *state, char *a) {
  return cmd_deliver(state, a);
}

int cmd_drop(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_bounce(struct fstate *state, char *a) {
  if (strlen(a)>ERRMSGMAX-128) {
    doerr(CGS(12, "bounce message too long (%d characters maximum)"),
	      ERRMSGMAX-128);
  }
  return FILTER_CONTINUE;
}

int cmd_moved(struct fstate *state, char *a) {
  if (strlen(a)>ERRMSGMAX-128) {
    doerr(CGS(12, "moved message too long (%d characters maximum)"),
	      ERRMSGMAX-128);
  }
  return FILTER_CONTINUE;
}

int cmd_trash(struct fstate *state, char *a) {
  return cmd_bounce(state, a);
}

int cmd_forward(struct fstate *state, char *a) {
  char *rest;
  char *unquoted;
  if (*a==0) {
    doerr(CGS(3, "missing forward address"));
  }
  if ((unquoted=unquote_addr821(a, 0, &rest))==NULL || rest==0 || *rest!=0)
    doerr(CGS(8, "bad address format <%s>"), a);
  free(unquoted);
  return FILTER_CONTINUE;
}

int cmd_forward2(struct fstate *state, char *a) {
  return cmd_forward(state, a);
}

int cmd_vacation(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_trace(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_score(struct fstate *state, char *a) {
  char *a0=a;
  switch(*a) {
  case '+':
  case '-':
    readscore(state, &a);
    break;
  case '*':
  case '/':
  case '%':
  case '=':
    a++;
  default:
    readscore(state, &a);
    break;
  }
  if (*a) {
    doerr(CGS(4, "bad score '%s'"), a0);
  }
  return FILTER_CONTINUE;
}


int cmd_skipto(struct fstate *state, char *a) {
  if (!*a) {
    doerr(CGS(5, "missing tag in skipto"));
  }
  if (strlen(a)>TAGMAX) {
    doerr(CGS(11, "tag name too long (%d characters max)"),TAGMAX);
  }
  return FILTER_CONTINUE;
}

int cmd_logrotate(struct fstate *state, char *a) {
  char *rest;
  strtol(a, &rest, 10);
  if (*rest) {
    doerr(CGS(13, "bad hour number (%s)"), a);
  }
  return FILTER_CONTINUE;
}

int cmd_timezone(struct fstate *state, char *a) {
  if (*a==':' || *a == '/' || strchr(a, '.'))
    doerr(CGS(14, "invalid time zone name (%s)"), a);
  return FILTER_CONTINUE;
}

#if 0
int cmd_module(struct fstate *state, char *a) {
  if (*a==0 || strchr(a, '/') || *a=='.')
    doerr(CGS(15, "invalid module name (%s)"), a);
  return FILTER_CONTINUE;
}

int cmd_return(struct fstate *state, char *a) {
  if (*a) {
    char *a0=a;
    readscore(state, &a);
    if (*a) {
      doerr(CGS(4, "bad score '%s'"), a0);
    }
  }
  return FILTER_CONTINUE;
}
#endif

int cmd_add(struct fstate *state, char *a) {
  if (!isheader(a))
    doerr(CGS(19, "invalid header '%s'"), a);
  return FILTER_CONTINUE;
}

int cmd_blacklist(struct fstate *state, char *a) {
  while(*a) {
    if (isspace((int)*a))
      break;
    a++;
  }
  while(*a && isspace((int)*a))
    a++;
  if (*a && !isheader(a)) {
    doerr(CGS(19, "invalid header '%s'"), a);
  }
  return FILTER_CONTINUE;
}

int cmd_extension(struct fstate *state, char *a) {
  if (*a) {
    return cmd_deliver(state, a);
  }
  return FILTER_CONTINUE;
}

int cmd_stow(struct fstate *state, char *a) {
  if (*a) {
    return cmd_deliver(state, a);
  }
  return FILTER_CONTINUE;
}

int cmd_log(struct fstate *state, char *a) {
  char *ret = readlogtype(state, a);
  if (ret) {
    doerr(CGS(20, "unknown logging capability '%s'"), ret);
  }
  return FILTER_CONTINUE;
}
int cmd_syslog(struct fstate *state, char *a) {
  char *a0 = a;
  if (*a) {
    if (strstr(a, "..") ||
	strstr(a, ".") == NULL ||
	a[strlen(a)-1] == '.') {
      doerr(CGS(20, "invalid host name '%s'"), a0);
    } else {
      while(*a) {
	if (!(isalnum((int)*a) || *a == '-' || *a == '.')) {
	  doerr(CGS(20, "invalid host name '%s'"), a0);
	  break;
	}
	a++;
      }
    }
  }
  return FILTER_CONTINUE;
}

int cmd_default(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_antivirus(struct fstate *state, char *a) {
  char *s = a;
  if (*s == '-') {
    s++;
  }
  while(*s) {
    if (*s == ':') {
      *s = 0;
      break;
    }
    if (*s<33 || *s>126) {/* RFC 2822, 3.6.8 */
      s = NULL;
      break;
    }
    s++;
  }
  if (s == NULL) {
    if (*a == '-') a++;
    doerr(CGS(19, "invalid header '%s'"), a);
  }
  return FILTER_CONTINUE; /* TODO check *a */
}

int cmd_spamassassin(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_pipe(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_debug(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_stop(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_header_set(struct fstate *state, char *a) {
  if (get_set(*a) == 0)
    doerr(CGS(6, "invalid set name '%c'"), *a);
  return FILTER_CONTINUE;
}

int unknown_cmd(struct fstate *state, char *cmd, char *a) {
  if (*cmd == 0) {
    if (*a) 
      doerr(CGS(6, "unknown command '%s'"), a);
    else
      doerr(CGS(10, "command missing"));
  } else {
    doerr(CGS(6, "unknown command '%s'"), cmd);
  }
  return FILTER_CONTINUE;
}

int default_cmd(struct fstate *state) {
  return FILTER_CONTINUE;
}

int cmd_message(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_ok(struct fstate *state, char *a) {
  return FILTER_CONTINUE;
}

int cmd_mod(struct fstate *state, char *a) {
  char *module = a;
  while(*a>=33 && *a<=126 && *a!=':') a++;
  if (isspace((int)*a))
    *a = 0;
  if (*module==0 || strchr(module, '/') || *module=='.' || *a) {
    doerr(CGS(15, "invalid module name (%s)"), a);
    return FILTER_CONTINUE;
  }
  while(isspace((int)*a)) *a++=0;
  if (*a) { /* optional header */
    char *head = a;
    while(*a) {
      if (*a == ':' || isspace((int)*a)) {
	a++;
	while(isspace((int)*a)) a++;
	if (*a)
	  doerr(CGS(19, "invalid header '%s'"), head);
	break;
      }
      if (*a<33 || *a>126) {
	doerr(CGS(19, "invalid header '%s'"), head);
	break;
      }
      a++;
    }
  }
  return FILTER_CONTINUE;
}

int main(int argc, char **argv) {
  struct fstate state = { 0, };
  /*  memset(&state, 0, sizeof(state)); */

  catd=catopen("filtercheck", 0);

  if (argc != 2 && argc !=3) {
    fprintf(stderr, "usage: filtercheck [-m] file\n");
    exit(1);
  }

  if (strcmp(argv[1], "-V") == 0) {
    fprintf(stderr, "filtercheck version %s\n", PACKAGE_VERSION);
    return EXIT_SUCCESS;
  } else if (strcmp(argv[1], "-m") == 0) {
    extern struct cmd mod_cmdlist[];
    state.in_module ++;
    re_init();
    filterfile_cmd(&state, argv[2], mod_cmdlist, unknown_cmd);
  } else {
    extern struct cmd cmdlist[];
    re_init();
    filterfile_cmd(&state, argv[1], cmdlist, unknown_cmd);
  }
  if (haserror)
    return EXIT_FAILURE;
  else 
    return EXIT_SUCCESS;

}
