/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#if WITH_PERL
static char const rcsid[] =
"$Id: perl.c,v 1.11 2003/02/11 16:02:47 lwa Exp $";

#include <EXTERN.h>
#include <perl.h>
#include <XSUB.h>

#include <fcntl.h>

#include <syslog.h>
#include <stdarg.h>
#include <locale.h>

#include "lmtp.h"

static PerlInterpreter *my_perl = NULL;


/* for older perl */
#ifndef pTHX_
#define pTHX_
#endif

#if !defined(call_pv) && HAVE_PERL_CALL_PV
#define call_pv perl_call_pv
#endif

static int fperl_is_safe(struct fstate *state, SV *var, char *what) {
  if (SvTYPE(var) != SVt_PV && SvTYPE(var) != SVt_PVMG ) {
    push_action_log(state, "ignored (%s is not a scalar (%d))",
		    what, SvTYPE(var) );
    return 0;
  }
  if (SvTAINTED(var)) {
    push_action_log(state, "ignored (%s is tainted)", what);
    return 0;
  }
  return 1;
}

static void fperl_do(struct fstate *state, char *err, int final) {
  switch(state->error[0]) {
  case '4':
    if (err[0] != '4') memcpy(err, state->error, sizeof(state->error));
    break;
  case '2':
    err[0] = 0;
    break;
  case '5':
    break;
  default:
    err[0] = 0;
    break;
  } 
}

#if 0
XS(xs_size) {
  dXSARGS;
  XSRETURN_IV(perlstate->bodysize);
}


XS(xs_xbounce) {
  dXSARGS;
  STRLEN n_a;
  SV *arg = sv_newmortal();
  int i;
  for (i = 0; i<items; i++) {
    sv_catsv(arg, ST(i));
  }
  // fprintf(stderr, "deliver final=%d\n", final);
  final = cmd_bounce(perlstate, SvPV(arg, n_a));
  XSRETURN_YES;
}
#endif

EXTERN_C void boot_DynaLoader(pTHX_ CV *cv);

EXTERN_C void xs_init(void) {
  char *file = __FILE__;
  newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
}  

int fperl_init(struct fstate *state) {
  if (state->perl) {
    char *argv[20];
    int argc = 0;
    argv[argc++] = "";
    argv[argc++] = "-T";
    argv[argc++] = "--";
    argv[argc++] = state->perl_filtername;
    argv[argc] = NULL;
    if ((my_perl = perl_alloc())== NULL) {
      syslog(LOG_ERR, "unable to perl_alloc()");
      return -1;
    }
    perl_construct(my_perl);
    if (perl_parse(my_perl, xs_init, argc, argv, NULL) != 0) {
      syslog(LOG_ERR, "unable to parse %s", state->perl_filtername);
      return -1;
    }
    
    setlocale(LC_ALL, ""); /* reset locale */
    
    //  newXS("size", xs_size, __FILE__);
    //  newXS("deliver", xs_deliver, __FILE__);
    //  newXS("bounce", xs_bounce, __FILE__);
  }
  return 0;
}


int filterperl(struct fstate *state, char *file) {
  int exitcode;
  SV *ret;
  HV *hret = NULL;
  STRLEN n_a;
  dSP;
  int count;
  int fd;
  int final = FILTER_CONTINUE;
  char err[ERRMSGMAX+1];

  if (!state->perl) /* should log something */
    return final;

  if ((fd = open(file, O_RDONLY)) == -1)
    return final;

  state->error[0] = '\0';
  err[0] = '\0';

  ENTER;
  SAVETMPS;
  PUSHMARK(SP);
  XPUSHs(sv_2mortal(newSVpv(state->rcpt->mbox, 0)));
  XPUSHs(sv_2mortal(newSVpv(file, 0)));
  XPUSHs(sv_2mortal(newSViv(fd)));
  { 
    struct headers *h = state->hdrlist; 
    AV *headers;
    HV *mail = newHV(); 
    headers = newAV();
    while(h) {
      SV *elt = newSVpvn(h->header, h->len);
      SvTAINTED_on(elt);
      av_push(headers, elt);
      h = h->next;
    }
    hv_store(mail, "headers", sizeof("headers")-1,
	     newRV_noinc((SV*)headers), 0);
    hv_store(mail, "size", sizeof("size")-1, newSViv(state->bodysize), 0);
    hv_store(mail, "lines", sizeof("lines")-1, newSViv(state->bodysize), 0);
    XPUSHs(newRV_noinc(sv_2mortal((SV*)mail)));
  }
  PUTBACK ;

  count = perl_call_pv(state->perl_eval, G_SCALAR|G_EVAL);
  close(fd);

  SPAGAIN;

  if (count != 1)
    croak("Big trouble\n") ;
 
  ret = POPs;

  if (SvTYPE(ret) == SVt_PV && fperl_is_safe(state, ret, "die")) {
    char *msg = SvPV(ret, n_a);
    if (*msg)
      push_action_log(state, "died (%s)", SvPV(ret, n_a));
    return FILTER_CONTINUE;
  }

  if (SvROK(ret))
    hret = (HV *)SvRV(ret);
  if (hret && SvTYPE(hret) == SVt_PVHV) {
#define FETCH(x) hv_fetch(hret, x, sizeof(x) -1, FALSE)
    SV **bounce_p = FETCH("bounce");
    SV **deliver_p = FETCH("deliver");
    SV **forward_p = FETCH("forward");
    SV **vacation_p = FETCH("vacation");
#undef FETCH 
    if (forward_p) {
      final = FILTER_STOP;
      if (SvROK(*forward_p)) {
	SV *forward;
	AV *forward_a = (AV *)SvRV(*forward_p);
	if (SvTYPE(forward_a) == SVt_PVAV)
	  while((forward = av_pop(forward_a)) != &PL_sv_undef) {
	    if (fperl_is_safe(state, forward, "forward"))
	      fperl_do(state, err, cmd_forward2(state, SvPV(forward, n_a)));
	  }
      } else {
	if (fperl_is_safe(state, *forward_p, "forward"))
	  fperl_do(state, err, cmd_forward2(state, SvPV(*forward_p, n_a)));
      }
    }
    if (deliver_p) {
      final = FILTER_STOP;
      if (vacation_p) 
	state->vacation = strdup(SvPV(*vacation_p, n_a));
      if (SvROK(*deliver_p)) {
	AV *deliver_a = (AV *)SvRV(*deliver_p);
	SV *deliver;
	if (SvTYPE(deliver_a) == SVt_PVAV)
	  while((deliver = av_pop(deliver_a)) != &PL_sv_undef) {
	    if (fperl_is_safe(state, deliver, "deliver"))
	      fperl_do(state, err, cmd_deliver(state, SvPV(deliver, n_a)));
	  }
      } else {
	if (fperl_is_safe(state, *deliver_p, "deliver")) 
	  fperl_do(state, err, cmd_deliver(state, SvPV(*deliver_p, n_a)));
      }
    }
    if (err[0]) {
      memcpy(state->error, err, sizeof(state->error));
    } else if (bounce_p) {
      final = FILTER_STOP;
      if (fperl_is_safe(state, *bounce_p, "bounce"))
	cmd_bounce(state, SvPV(*bounce_p, n_a));
    }

  }

  PUTBACK ;
  FREETMPS ;
  LEAVE ;

  fprintf(stderr, "final=%d\n", final);
  return final;
}

#endif
