/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: quotadb_link.c,v 1.6 2003/01/02 16:48:43 lwa Exp $";

#include "config.h"

#include <unistd.h>
#include <limits.h>
#include <syslog.h>

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>

#include "lmtp.h"

static void quotadb_link_file(struct fstate *state, char buffer[MAXPATHLEN], 
			       char *user) {
  char *s = buffer;
  char *linkdir = state->quota_db_linkdir;
  
  while(*linkdir) {
    *s ++ = *linkdir++;
  }
  *s ++ = '/';
  *s ++ = '_'; /* avoid ".." */
  while(*user && s < buffer + MAXPATHLEN-4) {
    switch(*user) {
    case '/':
      *s ++ = '='; *s ++ = '2'; *s ++ = 'f'; break;
    case '=':
      *s ++ = '='; *s ++ = '3'; *s ++ = 'd'; break;
    case ' ':
      *s ++ = '='; *s ++ = '2'; *s ++ = '0'; break;
    default:
      if (isupper((int)*user)) {
	*s++ = tolower(*user);
      } else {
	*s++ = *user;
      }
    }
    user ++;
  }
  *s = 0;
}

void quotadb_link_add(struct fstate *state, char *user, int code) {
  char buffer[MAXPATHLEN];
  char value[1000];
  /*  fprintf(stderr, "add\n"); */

  quotadb_link_file(state, buffer, user);

  snprintf(value, sizeof(value), "%lu %d",
	   (unsigned long)state->date, code);
  value[sizeof(value)-1] = 0;
  symlink(value, buffer);
}

void quotadb_link_del(struct fstate *state, char *user) {
  char buffer[MAXPATHLEN];
  /*  fprintf(stderr, "del\n"); */
  quotadb_link_file(state, buffer, user);
  unlink(buffer);
}

char *quotadb_link_check(struct fstate *state, char *user) {
  char buffer[MAXPATHLEN];
  char value[1000];
  char *value_p;
  int len;
  unsigned long date;
  int code;
  int num;
  /*  fprintf(stderr, "check\n"); */
  
  quotadb_link_file(state, buffer, user);
  
  if ((len = readlink(buffer, value, sizeof(value)-1))==-1)
    return NULL;
  value[len] = 0;
  value_p = value;

  date = strtoul(value_p, &value_p, 10);
  code = strtol(value_p, &value_p, 10);
  num = strtol(value_p, &value_p, 10);
  if (date && code) {
    if (state->date - date < state->quota_db_timeout) {
      num ++;
      if (state->quota_maxchecks > 0 && num > state->quota_maxchecks)
	code = 552;
      snprintf(value, sizeof(value), "%lu %d %d",
	       (unsigned long)state->date, code, num);
      value[sizeof(value)-1] = 0;
      unlink(buffer);
      symlink(value, buffer);
      return seterr(state, code, "4.2.2",
		    "<%s>: Mailbox full -- Quota exceeded", user);
    } else {
      unlink(buffer);
    }
  } else {
    syslog(LOG_ERR, "error quotadb_link: invalid format in '%s'", buffer);
  }
  return NULL;
}

int quotadb_link_open(struct fstate *state) {
  if (state->quota_db_linkdir == NULL) {
    syslog(LOG_ERR, "quotadb_link: undefined link directory");
    return -1;
  }
  return 0; /* do nothing */
}
void quotadb_link_close(struct fstate *state) {
  return; /* do nothing */
}

