/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "config.h"

#if !WITH_DB
#undef WITH_REGEXP_DB_CACHE
#endif

#if WITH_REGEXP_DB_CACHE

static char const rcsid[] =
"$Id: recache.c,v 1.4 2004/03/24 15:33:07 lwa Exp $";

int re_cache = 0;

#include <db.h>
#include <stdio.h>
#include "re.h"

static DB *re_db = NULL;

struct re_cache_data {
  int          (*exec)(struct _re *re, char *string);
  void         (*close)(struct _re *re);
  void *code;
  void *hint; /* for pcre */
  unsigned int usage;
};


char *re_cache_init(void) {
  if (re_cache == 0) {
    int ret = db_create(&re_db, NULL, 0);
    if (ret != 0)
      return db_strerror(ret);
#if DB_VERSION_MAJOR == 3 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR == 0)
    ret = re_db->open(re_db, NULL, NULL, DB_HASH, DB_NOMMAP, 0);
#else
    ret = re_db->open(re_db, NULL, NULL, NULL, DB_HASH, DB_NOMMAP, 0);
#endif
    if (ret != 0) {
      re_db->close(re_db, 0);
      return db_strerror(ret);
    }
    re_cache = 1;
    return NULL;
  }
  return NULL;
}

static void re_cache_close(RE *re) {
  return;
}

RE *re_cache_search(RE *re, char *source, char *source_end, 
		    char *pattern, char *pattern_end, int icase) {
  DBT key = { 0, }, value = {0, };
  static struct re_cache_data red;
  static RE xre;
  int ret;

  if (re_db == NULL) 
    re_cache_init();

  key.data = source;
  key.size = source_end - source;
  key.ulen = key.size;

  value.data = &red;
  value.flags = DB_DBT_USERMEM;
  value.ulen = sizeof(red);
  
  ret = re_db->get(re_db, NULL, &key, &value, 0);
  /*
    printf("re(%.*s): %s\n", (int)(source_end - source), source,
	 db_strerror(ret));
  */
  if (ret == DB_NOTFOUND) {
    char saved_char = *pattern_end;
    *pattern_end = 0;
    re->compile(re, pattern, icase);
    *pattern_end = saved_char;
    red.exec = re->exec;
    red.code = re->code;
    red.hint = re->hint;
    red.usage = 0;
    value.data = &red;
    value.size = sizeof(re);
    value.flags = 0;
    ret = re_db->put(re_db, NULL, &key, &value, 0);
  }
  xre.compile = NULL;
  xre.exec = red.exec;
  xre.close = re_cache_close;
  xre.code = red.code;
  xre.hint = red.hint;
  red.usage ++;
  return &xre;
}

#endif
