/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: slist.c,v 1.10 2003/11/10 15:52:25 lwa Exp $";

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "lmtp.h"

#if HAVE_MERGESORT
# define SORT mergesort
#else
# define SORT qsort
#endif


int slist_casecmp(const void *a, const void *b) {
  return strcasecmp(((struct slist_elt *)a)->key,
		    ((struct slist_elt *)b)->key);
}

int slist_cmp(const void *a, const void *b) {
  return strcmp(((struct slist_elt *)a)->key,
		((struct slist_elt *)b)->key);
}

char *slist_get_value(struct slist *m, char *key) {
  struct slist_elt *ret = slist_search(m, key);
  if (ret) {
    /*    fprintf(stderr, "search %s => %s\n", key, ret->value);  */
    return ret->value;
  } else {
    /*    fprintf(stderr, "search %s not found!\n", key); */
    return NULL;
  }
}

struct slist_elt *slist_search(struct slist *m, char *key) {
  struct slist_elt skey;
  struct slist_elt *ret;
  if (m == NULL || m->nmemb == 0)
    return NULL;
  if (!m->sorted) {
    SORT(m->elts, m->nmemb, sizeof(struct slist_elt), m->compar);
    m->sorted=1;
  }
  skey.key=key;
  ret=bsearch(&skey, m->elts, m->nmemb,
	      sizeof(struct slist_elt), m->compar);
  return ret;
}

void slist_reset(struct slist *m) {
  int idx;
  if (m) {
    for(idx=0; idx<m->nmemb; idx++) {
      m->elts[idx].flags=0;
      if (m->elts[idx].extra)
	free(m->elts[idx].extra);
      m->elts[idx].extra=NULL;
    }  
  }
}

void slist_dump(struct slist *m, FILE *f) {
  int idx;
  if (!m) {
    fprintf(f, "*** DUMP empty\n");
    return;
  }
  if (!m->sorted) {
    SORT(m->elts, m->nmemb, sizeof(struct slist_elt), m->compar);
    m->sorted=1;
  }
  fprintf(f, "*** DUMP of %p\n", m);
  for(idx=0; idx<m->nmemb; idx++) {
    fprintf(f, "%s=%s (%X)\n", 
	    m->elts[idx].key, m->elts[idx].value, m->elts[idx].flags);
  }  
  fprintf(f, "***\n");
}

struct slist *slist_put(struct slist *m, char *key, char *value) {
  int idx;
  struct slist_elt skey;
  int key_found=0;
  if (m==NULL) {
    m = malloc(sizeof(struct slist));
    m->nmemb  = 0;
    m->compar=slist_casecmp;
    m->elts = NULL;
    m->from_malloc = 1;
  }
  if (m==NULL)
    return NULL;
  skey.key=key;
  m->sorted = 0;
  for(idx=0; idx<m->nmemb; idx++) {
    if (m->compar(&(m->elts[idx]), &skey)==0) {
      key_found = 1;
      free(m->elts[idx].value);
      free(m->elts[idx].key);
      free(m->elts[idx].extra);
      if (value==NULL) {
	m->nmemb--;
	if (idx<m->nmemb) {
	  m->elts[idx].value = m->elts[m->nmemb].value;
	  m->elts[idx].key   = m->elts[m->nmemb].key;
	  m->elts[idx].flags = m->elts[m->nmemb].flags;
	  m->elts[idx].extra = m->elts[m->nmemb].extra;
	}
      } else {
	if ((m->elts[idx].key   = strdup(key))==NULL)  return NULL;
	if ((m->elts[idx].value = strdup(value))==NULL) {
	    free(m->elts[idx].key);
	    return NULL;
	}
	m->elts[idx].flags = 0;
	m->elts[idx].extra = NULL;
      }
      break;
    }
  }
  if (!key_found) {
    struct slist_elt *melts;
    melts = realloc(m->elts, 
		    (m->nmemb+1)*sizeof(struct slist_elt));
    if (melts==NULL)
      return NULL;
    m->elts=melts;
    if (value) {
      if ((m->elts[m->nmemb].key   = strdup(key))==NULL) return NULL;
      if ((m->elts[m->nmemb].value = strdup(value))==NULL) {
	free(m->elts[m->nmemb].key);
	return NULL;
      }
    }
    m->elts[idx].flags = 0;
    m->elts[idx].extra = NULL;
    m->nmemb++;
  }
  return m;
}

void slist_clean(struct slist *m) {
  if (m) {
    int idx = 0;
    for (idx = 0; idx < m->nmemb; idx ++) {
      free(m->elts[idx].key);
      free(m->elts[idx].value);
      if (m->elts[idx].extra) 
	free(m->elts[idx].extra);
    }
    if (m->from_malloc)
      free(m);
  }
}
