/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: strfmt.c,v 1.6 2003/01/02 16:48:43 lwa Exp $";

#include "config.h"

#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#define TYPE_INTEGER 1
#define TYPE_STRING  2

struct reg {
  int type;
  union {
    char *string;
    int integer;
  } val;
};

char *strfmt(char *buf, int buflen, char *fmt, char *val, ...) {
  struct reg regs[26]={{0,},};
  char *obuf = buf;

  va_list ap;
  va_start(ap, val);
  while(*val) {
    int type = TYPE_INTEGER;
    if (*val=='$') {
      type = TYPE_STRING;
      val ++;
    }
    if (*val >= 'a' && *val <='z') {
      int r = *val - 'a';
      regs[r].type = type;
      switch(type) {
      case TYPE_INTEGER:
	regs[r].val.integer=va_arg(ap, int);
	break;
      case TYPE_STRING:
	regs[r].val.string=va_arg(ap, char *);
	break;
      }
    }
    val ++;
  }

  while(*fmt && buflen > 0) {
    int srclen;
    char *src;
    char sbuf[1024];
    if (*fmt == '%') {
      int r=-1;
      int type = 0;
      fmt++;
      if (*fmt >= 'A' && *fmt <= 'Z') {
	r = *fmt - 'A';
      } else if (*fmt >= 'a' && *fmt <= 'z') {
	r = *fmt - 'a';
      }
      if (r>=0) 
	type = regs[r].type;

      switch(type) {
      case TYPE_INTEGER:
	srclen=sprintf(sbuf, "%d", regs[r].val.integer);
	src = sbuf;
	break;
      case TYPE_STRING:
	if (regs[r].val.string) {
	  srclen = strlen(regs[r].val.string);
	  src = regs[r].val.string;
	} else {
	  srclen = 0;
	  src = "";
	}
	break;
      default:
	srclen = 2;
	src = fmt - 1;
	break;
      }
      fmt++;
    } else {
      src = fmt;
      while(*fmt && *fmt != '%') 
	fmt++;
      srclen = fmt - src;
    }
    buflen -= srclen;
    if (srclen) {
      if (buflen > 0) {
	memcpy(buf, src, srclen);
	buf+=srclen;
      } else {
	break;
      }
    }
  }
  *buf=0;
  if (buflen<0)
    return NULL;
  else {
    return obuf;
  }
}

#if DEBUG_STRFMT

int main(int argc, char **argv) {
  char buf[40]={0,};
  char *r;
  r = strfmt(buf, sizeof(buf), "X-Foo: %x+%y+%a", "a$x", 10000, "errrr");
  printf("r=%p, %s\n", r, buf);
}

#endif
