/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: tools.c,v 1.28 2003/11/20 13:31:41 lwa Exp $";

#include "config.h"

#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>

#if !HAVE_VSYSLOG
#include <syslog.h>
#endif

#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "lmtp.h"

void mkbox(struct fstate *state, char *box) {
  char dir[PATH_MAX+1];
  if (*box) {
    snprintf(dir, PATH_MAX, ".%s", box);
    mkdir(dir, 0755);
    snprintf(dir, PATH_MAX, ".%s/maildirfolder", box);
    close(open(dir, O_WRONLY|O_CREAT, 0444));
    snprintf(dir, PATH_MAX, ".%s/tmp", box);
    mkdir(dir, 0755);
  }
  snprintf(dir, PATH_MAX, ".%s/new", box);
  mkdir(dir, state->mode_mail);
  snprintf(dir, PATH_MAX, ".%s/cur", box);
  mkdir(dir, state->mode_mail);
}

char *lower(char *s) {
  char *s0 = s;
  while(*s) {
    *s = tolower(*s);
    s++;
  }
  return s0;
}

int opentmp(struct fstate *state, char tmpname[PATH_MAX]) {
  int tmpfd;
  snprintf(tmpname, PATH_MAX-1, "tmp/%lu.P%luQ%lu.%s:1,lmtp-XXXXXX",
	   (unsigned long)state->date, (unsigned long)state->pid,
	   state->counter, state->myhostname);
  tmpname[PATH_MAX-1]=0;
  if ((tmpfd=mkstemp(tmpname))==-1) {
    if (errno == ENOENT) {
      if (mkdir("tmp", state->mode_mail) == -1) {
	if (errno != EEXIST)
	  return -1;
      }
      tmpfd=mkstemp(tmpname);
    }
  }
  return tmpfd;
}


char *getuserdomain(char *user, char domain[255]) {
  char *x;
  if ((x=strrchr(user, '@'))!=NULL) {
    char *s=domain;
    x++;
    while(*x) {
      if ((isascii((int)*x) && isalnum((int)*x)) || *x=='.' || *x=='-')
	*s++=isupper((int)*x) ? tolower(*x) : *x;
      x++;
    }
    *s='\0';
  } else {
    *domain='\0';
  }
  return domain;
}

FILE *opentemplate(char *path, char *domain) {
  char filename[PATH_MAX];
  FILE *msg;

  if (path ==NULL && *path == 0)
    return NULL;

  if (*domain == 0)
    return fopen(path, "r");

  snprintf(filename, sizeof(filename), "%s%s", path, domain);
  if ((msg = fopen(filename, "r"))==NULL) {
    if (errno==ENOENT) /* file does not exist */
      return fopen(path, "r");
  }
  return msg;
}

FILE *newmessage(struct fstate *state, char *type, char tmpname[PATH_MAX]) {
  FILE *tmp;
  int tmpfd;
  char bufdate[100];

  tmpfd=opentmp(state, tmpname);
  if (tmpfd<0) {
    return NULL;
  }
  if ((tmp = fdopen(tmpfd, "w"))==NULL ) {
    unlink(tmpname);
    close(tmpfd);
    return NULL;
  }

  strftime(bufdate, sizeof(bufdate)-1, STRFTIME_FMT,
	   localtime(&(state->date)));

  fprintf(tmp, "Return-Path: <>\n");
  fprintf(tmp, "Delivered-To: %s\n", state->qrecipient);
  fprintf(tmp,
	  "Received: from localhost by %s (lmtpd) with %s id %lu.%03lu;\n"
	  "\t%s\n",
	  state->myhostname, type,
	  (unsigned long)state->pid, state->counter,
	  bufdate);
  fprintf(tmp, "Message-ID: <%lX-" MID_FMT  ".%s@%s>\n",
	  (long)state->date, state->counter, type, state->myhostname);
  fprintf(tmp, "Date: %s\n", bufdate);

  return tmp;
}

int endmessage(struct fstate *state, char *type, char *tmpname, FILE *tmp) {
  char *dst;
  unsigned long total_size=0;
  if (fsync(fileno(tmp))==-1 && errno != EINVAL) {
    fclose(tmp);
    unlink(tmpname);
    return EOF;
  }
  total_size=ftell(tmp);
  if (fclose(tmp)==EOF) {
    unlink(tmpname);
    return EOF;
  }

  dst = state->default_dest ? state->default_dest : "";
  
  if (strchr(dst, '@')==NULL) {
    char filename[PATH_MAX];
    getbox(dst);  
    snprintf(filename, PATH_MAX, ".%s/new/%lu.P%luQ%lu.%s-%s,S=%lu", dst,
	     (unsigned long)state->date, (unsigned long)state->pid,
	     state->counter, state->myhostname, type,
	     (unsigned long)total_size);

    if (rename(tmpname, filename)==-1) {
      if (errno == ENOENT) {
	mkbox(state, dst);
	if (rename(tmpname, filename)==-1) {
	  unlink(tmpname);
	  return EOF;
	}
      }
      unlink(tmpname);
      return EOF;
    }
    mdq_add(state->quotainfo, (long)total_size, +1);
  } else {
    FILE *f = fopen(tmpname, "r");
    if (f) {
      int fd = runsendmail(NULL, state->sendmail_path, "", dst);
      if (fd != -1) {
	int len;
	char buf[8192];
	while((len = fread(buf, 1, sizeof(buf), f))>0)
	  write(fd, buf, len);
	fclose(f);
      }
      waitsendmail(NULL, fd);
      unlink(tmpname);
    }
  }

  return 0;
}

/* compute CR & LF in headers */
char *purify_header(char *src, char *content, int len) {
  char *c = content;
  char *content_end = content+len-3;
  while(*src && c < content_end) {
    if ((*src == '\n' || *src == '\r')) {
      while (*src == '\n' || *src == '\r') src++;
      if (*src) {
	*c++='\n';
	if (*src != ' '  || *src == '\t') 
	  *c++=' ';
      }
    } else {
      *c++=*src++;
    }
  }
  *c++=0;
  return content;
}



#if !HAVE_VSYSLOG
/* fake function for HPUX */
void vsyslog(int priority, const char *message, va_list args) {
  char buf[10000]; /* big buffer */
  int len;
  len = vsnprintf(buf, sizeof(buf) -1, message, args);
  buf[sizeof(buf) -1] = 0;
  syslog(priority, "%s", buf);
}
#endif

/* copy and split string in argv[] format returns argc */

int split_args(char *cmd, char *args, size_t arglen,
	       char *argv[], int argcmax) {
  int argc = 0;
  char *args_c = args;

  while(*cmd) {
    while(*cmd == ' ' || *cmd == '\t')
      cmd ++;
    if (*cmd) {
      argv[argc++] = args_c;
      while(*cmd && *cmd != ' ' && *cmd != '\t') {
        if (*cmd == '\\')
          if (cmd[1]) cmd ++;
        *args_c ++ = *cmd ++;
      }
      *args_c++ = 0;
    }
  }
  argv[argc] = NULL;
  return argc;
}
